from .turtle.polygon_turtle import *
from .turtle.turtle import *

class LSystem:
    def __init__(self):
        self.start_word = None
        self.word = None
        self.rules = None
    def iterate(self, n):
        for k in range(n):
            new_word = []
            for i, w in enumerate(self.word):
                aux = self.rules(self.word, i)
                for a in aux:
                    new_word.append(a)
            self.word = new_word
    def reset(self):
        self.word = start_word
    def to_vec_instr(self):
        raise NotImplementedError('call to abstract method')

# Example - Barnesley fern
class BarnFern(LSystem):
    def __init__(self):
        self.start_word = ['X']
        self.word = self.start_word
        def rules(word, i):
            dic_rules = {
                'X': list('F+[[X]-X]-F[-FX]+X'),
                'F': list('FF'),
                '+' : '+',
                '-' : '-',
                '[':'[', 
                ']':']'}
            return dic_rules[word[i]]
        self.rules = rules
    def to_vec_instr(self):
        instr = []
        switcher = {
            'F':Df(1),
            '+':Rot(mathutils.Quaternion((0, 0, 1), math.radians(25))), #left 25
            '-':Rot(mathutils.Quaternion((0, 0, 1), math.radians(-25))),
            '[':Push(),
            ']':Pop(),
        }
        for c in self.word:
            if switcher.get(c, None) is not None:
                instr.append(switcher[c])
        return instr
    def add_to_scene(self, turtle):
        stack = []
        pmesh = PMesh()
        Instruction.exec_all(turtle, stack, pmesh, self.to_vec_instr())
        pmesh.scale_max_coord(4)
        pmesh.add_to_scene("mesh barnFern", "obj barnFern")
        #bpy.context.space_data.context = 'MATERIAL'
        #bpy.ops.material.new()
        #bpy.context.object.active_material.name = "material barn"
        #bpy.data.node_groups["Shader Nodetree"].nodes["Emission"].inputs[1].default_value = 3

class RotBarnFern(BarnFern):
    def to_vec_instr(self):
        instr = []
        golden_rot = mathutils.Quaternion((1, 0, 0), math.radians(137.507))
        switcher = {
            'F':[Df(1), Rot(golden_rot)],
            '+':[Rot(mathutils.Quaternion((0, 0, 1), math.radians(25)) )], #left 25
            '-':[Rot(mathutils.Quaternion((0, 0, 1), math.radians(-25)))],
            '[':[Push()],
            ']':[Pop()],
        }
        for c in self.word:
            if switcher.get(c, None) is not None:
                for i in switcher[c]:
                    instr.append(i)
        return instr


  
